#! /usr/bin/env python
"""
Author: Jeremy M. Stober
Program: GMM.PY
Date: Friday, June 24 2011/Volumes/NO NAME/seds/nodes/gmm.py
Description: A python class for creating and manipulating GMMs.
"""

import scipy.cluster.vq as vq
import numpy as np
import numpy.linalg as la
import numpy.random as npr
import random as pr
npa = np.array

import sys; sys.path.append('.')
import pdb

#import matplotlib
import pylab
from normal import Normal

class GMM(object):

    def __init__(self, dim = None, ncomps = None, data = None,  method = None, filename = None, params = None):

        if not filename is None:  # load from file
            self.load_model(filename)

        elif not params is None: # initialize with parameters directly
            self.comps = params['comps']
            self.ncomps = params['ncomps']
            self.dim = params['dim']
            self.priors = params['priors']

        elif not data is None: # initialize from data

            assert dim and ncomps, "Need to define dim and ncomps."

            self.dim = dim
            self.ncomps = ncomps
            self.comps = []

            if method is "uniform":
                # uniformly assign data points to components then estimate the parameters
                npr.shuffle(data)
                n = len(data)
                s = n / ncomps
                for i in range(ncomps):
                    self.comps.append(Normal(dim, data = data[i * s: (i+1) * s]))

                self.priors = np.ones(ncomps, dtype = "double") / ncomps

            elif method is "random":
                # choose ncomp points from data randomly then estimate the parameters
                mus = pr.sample(data,ncomps)
                clusters = [[] for i in range(ncomps)]
                for d in data:
                    i = np.argmin([la.norm(d - m) for m in mus])
                    clusters[i].append(d)

                for i in range(ncomps):
                    print(mus[i], clusters[i])
                    self.comps.append(Normal(dim, mu = mus[i], sigma = np.cov(clusters[i], rowvar=0)))

                self.priors = np.ones(ncomps, dtype="double") / np.array([len(c) for c in clusters])

            elif method is "kmeans":
                # use kmeans to initialize the parameters
                (centroids, labels) = vq.kmeans2(data, ncomps, minit="points", iter=100)
                clusters = [[] for i in range(ncomps)]
                for (l,d) in zip(labels,data):
                    clusters[l].append(d)

                # will end up recomputing the cluster centers
                for cluster in clusters:
                    self.comps.append(Normal(dim, data = cluster))

                self.priors = np.ones(ncomps, dtype="double") / np.array([len(c) for c in clusters])

            else:
                raise ValueError("Unknown method type!")

        else:

            # these need to be defined
            assert dim and ncomps, "Need to define dim and ncomps."

            self.dim = dim
            self.ncomps = ncomps

            self.comps = []

            for i in range(ncomps):
                self.comps.append(Normal(dim))

            self.priors = np.ones(ncomps,dtype='double') / ncomps

    def __str__(self):
        res = "%d" % self.dim
        res += "\n%s" % str(self.priors)
        for comp in self.comps:
            res += "\n%s" % str(comp)
        return res

    def save_model(self):
        pass

    def load_model(self):
        pass

    def mean(self):
        return np.sum([self.priors[i] * self.comps[i].mean() for i in range(self.ncomps)], axis=0)

    def covariance(self): # computed using Dan's method
        m = self.mean()
        s = -np.outer(m,m)

        for i in range(self.ncomps):
            cm = self.comps[i].mean()
            cvar = self.comps[i].covariance()
            s += self.priors[i] * (np.outer(cm,cm) + cvar)

        return s

    def pdf(self, x):
        responses = [comp.pdf(x) for comp in self.comps]
        return np.dot(self.priors, responses)

    def condition(self, indices, x):
        """
        Create a new GMM conditioned on data x at indices.
        """
        condition_comps = []
        marginal_comps = []

        for comp in self.comps:
            condition_comps.append(comp.condition(indices, x))
            marginal_comps.append(comp.marginalize(indices))

        new_priors = []
        for (i,prior) in enumerate(self.priors):
            new_priors.append(prior * marginal_comps[i].pdf(x))
        new_priors = npa(new_priors) / np.sum(new_priors)

        params = {'ncomps' : self.ncomps, 'comps' : condition_comps,
                  'priors' : new_priors, 'dim' : marginal_comps[0].dim}

        return GMM(params = params)

    def em(self, data, nsteps = 100):

        k = self.ncomps
        d = self.dim
        n = len(data)

        for l in range(nsteps):

            # E step

            responses = np.zeros((k,n))

            for j in range(n):
                for i in range(k):
                    responses[i,j] = self.priors[i] * self.comps[i].pdf(data[j])

            responses = responses / np.sum(responses,axis=0) # normalize the weights

            # M step

            N = np.sum(responses,axis=1)

            for i in range(k):
                mu = np.dot(responses[i,:],data) / N[i]
                sigma = np.zeros((d,d))

                for j in range(n):
                   sigma += responses[i,j] * np.outer(data[j,:] - mu, data[j,:] - mu)

                sigma = sigma / N[i]

                self.comps[i].update(mu,sigma) # update the normal with new parameters
                self.priors[i] = N[i] / np.sum(N) # normalize the new priors


def shownormal(data,gmm):

    xnorm = data[:,0]
    ynorm = data[:,1]

    # Plot the normalized faithful data points.
    fig = pylab.figure(num = 1, figsize=(4,4))
    axes = fig.add_subplot(111)
    axes.plot(xnorm,ynorm, '+')

    # Plot the ellipses representing the principle components of the normals.
    for comp in gmm.comps:
        comp.patch(axes)

    pylab.draw()
    pylab.show()


if __name__ == '__main__':

    """
    Tests for gmm module.
    """


    # x = npr.randn(20, 2)

    # print "No data"
    # gmm = GMM(2,1,2) # possibly also broken
    # print gmm

    # print "Uniform"
    # gmm = GMM(2,1,2,data = x, method = "uniform")
    # print gmm

    # print "Random"
    # gmm = GMM(2,1,2,data = x, method = "random") # broken
    # print gmm

    # print "Kmeans"
    # gmm = GMM(2,1,2,data = x, method = "kmeans") # possibly broken
    # print gmm


    x = np.arange(-10,30)
    #y = x ** 2 + npr.randn(20)
    y = x + npr.randn(40) # simple linear function
    #y = np.sin(x) + npr.randn(20)
    data = np.vstack([x,y]).T


    gmm = GMM(dim = 2, ncomps = 4,data = data, method = "random")
    shownormal(data,gmm)

    gmm.em(data,nsteps=1000)
    shownormal(data,gmm)
    ngmm = gmm.condition([0],[-3])



# http://eyalarubas.com/python-subproc-nonblock.html
from threading import Thread


try:
    from Queue import Queue, Empty
except ImportError:
    from queue import Queue, Empty


class NonBlockingStreamReader:

    def __init__(self, stream):
        """
        stream: the stream to read from.
                Usually a process' stdout or stderr.
        """

        self._s = stream
        self._q = Queue()

        def _populateQueue(stream, queue):
            """
            Collect lines from 'stream' and put them in 'quque'.
            """

            while True:
                line = stream.readline()
                if line:
                    queue.put(line)
                else:
                    raise UnexpectedEndOfStream

        self._t = Thread(target = _populateQueue,
                args = (self._s, self._q))
        self._t.daemon = True
        self._t.start() #start collecting lines from the stream

    def readline(self, timeout = None):
        try:
            return self._q.get(block = timeout is not None,
                    timeout = timeout)
        except Empty:
            return None

class UnexpectedEndOfStream(Exception): pass




from biokit.rtools import package
from nose.plugins.attrib import attr
try:
    import create_dummy_package as dun
except:
    from . import create_dummy_package as dun


# This will not work on travis since you will need R


@attr('Ronly')
def test_install_packages():
    d = dun.CreateDummy()
    d()
    package.install_package('./dummy/dummytest_1.0.0.tar.gz', verbose=True) 
    d._clean()


@attr('Ronly')
def test_install_packages():
    package.install_package("truncnorm")


@attr('Ronly')
def test_get_r_version():
    package.get_R_version()


@attr('Ronly')
def test_bioclite():
    package.biocLite()
    package.biocLite('truncnorm')


@attr('Ronly')
def test_rpackage():
    # need to play with a pacakge. Again a dummy one would be handy
    p = package.RPackage('CellNOptR')
    p = package.RPackage('CellNOptR', version_required="2000.0")
    p = package.RPackage('CellNOptR', version_required="2000")
    print(p)

    p = package.RPackage('dummy')
    assert p.isinstalled is False
    print(p)


@attr('Ronly')
def test_pm():

    pm = package.RPackageManager()
    pm.packages
    pm.installed
    pm.available
    pm.get_package_version('base')


    try:
        pm.get_package_version('whatever_is_not_installed')
        assert False
    except:
        assert True


    pm.remove('truncnorm')
    pm.biocLite('truncnorm')
    pm.biocLite(['truncnorm'])
    pm.biocLite(None)


    d = dun.CreateDummy()
    d()
    pm.install('dummy/dummytest_1.0.0.tar.gz') 
    pm.remove('dummytest')
    d._clean()

    pm.remove('truncnorm')
    pm.install('truncnorm')
    pm.install('truncnorm') # trying again with required version




from sklearn.mixture import GMM
import numpy as np
import pandas as pd




class GMMSK(object):
    def __init__(self, means=[60,100,140], sigmas=[10,10,10], weights=[.1,.8,.1]):
        n_component = len(means)
        self.gmm = GMM(n_component, n_iter=100)
        self.gmm.means_ = np.array([[x] for x in means])
        self.gmm.covars_ = np.array([[x] for x in sigmas])**2
        self.gmm.weights_ = np.array(weights)

    def sample(self, N=10000):
        return self.gmm.sample(N)

    def fit(self, X, k=3):
        model = GMM(k).fit(X)
        return model

    def fit2(self, X, k=3):
        from biokit.stats import mixture
        self.mf = mixture.EM(X[:,0])
        self.mf.estimate(k=3)
        return self.mf.results
    
    def fit3(self, X, k=3):
        from biokit.stats import mixture
        self.mf = mixture.GaussianMixtureFitting(X[:,0])
        self.mf.estimate(k=3)
        return self.mf.results

    def error(self, n=10, N=10000):
        error_mu = []
        error_sigma = []
        error_weight = []
        for this in range(n):
            model = self.fit(self.sample(N))

            index = np.argmax(model.weights_)

            error = self.gmm.means_[1] - model.means_[index]
            error = 100*abs(error/self.gmm.means_[1])
            error_mu.append(error)

            error = np.sqrt(self.gmm.covars_[1]) - np.sqrt(model.covars_[index])
            error = 100*abs(error/np.sqrt(self.gmm.covars_[1]))
            error_sigma.append(error)

            error = self.gmm.weights_[1] - model.weights_[index]
            error = 100*abs(error/self.gmm.weights_[1])
            error_weight.append(error)
        df = pd.DataFrame({'mu': [x[0] for x in error_mu], 
            'sigma': [x[0] for x in error_sigma], 
            'weight': error_weight})
        df = df[['mu', 'sigma', 'weight']]
        return df

    def error_biokit(self, n=10, N=10000, mode='em'):
        # we look at the central distribution only
        error_mu = []
        error_sigma = []
        error_weight = []
        for this in range(n):
            if mode == "em":
                estimation = self.fit2(self.sample(N))
            else:
                estimation = self.fit3(self.sample(N))
            error = self.gmm.means_[1] - estimation.mus[1]
            error = 100*abs(error/self.gmm.means_[1])
            error_mu.append(error)

            error = np.sqrt(self.gmm.covars_[1]) - estimation.sigmas[1]
            error = 100*abs(error/np.sqrt(self.gmm.covars_[1]))
            error_sigma.append(error)
            
            error = self.gmm.weights_[1] - estimation.pis[1]
            error = 100*abs(error/self.gmm.weights_[1])
            error_weight.append(error)
        df = pd.DataFrame({'mu': [x[0] for x in error_mu], 
            'sigma': [x[0] for x in error_sigma], 
            'weight': error_weight})
        df = df[['mu', 'sigma', 'weight']]
        return df

    def error2(self, n=10, N=10000):
        return self.error_biokit(n, N)
    def error3(self, n=10, N=10000):
        return self.error_biokit(n, N, mode="gmf")






from biokit.viz import Heatmap
from biokit import biokit_data

def test_heatmap():
    filename = biokit_data("test_heatmap.csv")
    import pandas as pd
    data = pd.read_csv(filename, skiprows=2, index_col=0)

    h = Heatmap(data)
    h.plot(cmap='hot')
    h.row_method= 'single'
    h.col_method= 'single'
    #    category_cols=[0,0,1,1], 
    #    category_rows=[0,1,2,0,0,1,2,2,2,1])







def test1():
    import numpy as np
    fc = np.random.randn(1000)
    pvalue = np.random.randn(1000)
    from biokit import Volcano
    v = Volcano(fc, -np.log10(pvalue**2))
    v.plot(pvalue_threshold=3)





from biokit.io.fastq import FASTQ
from nose.plugins.attrib import attr



class test_FASTQ(object):
    @classmethod
    def setup_class(klass):
        klass.f = FASTQ()

    def set_example1(self):
        self.f.identifier = ''
        self.f.sequence = 'CCCC'
        self.f.quality = '@ABC'

    def set_example2(self):
        self.f.identifier = '@slicing'
        self.f.sequence = 'CCCCTTTT'
        self.f.quality = '@ABC;;;;'

    def test_offset(self):
        self.set_example1()
        assert self.f.get_quality_integer() == [31, 32, 33, 34]

        self.f.offset = 64
        assert self.f.get_quality_integer() == [0, 1, 2, 3]
        self.f.offset = 33

    def test_quality(self):
        assert self.f.get_quality(0.000000001) == 90

        assert self.f.error_probability_from_quality(90) == 1e-9
        assert self.f.quality_from_error_probability(1e-9) == 90

    def test_others(self):
        self.set_example1()
        #assert len(self.f) == 1
        print(self.f)
        self.f.check()

    def test_slicing(self):
        self.set_example2()
        newf = self.f[2:6]
        assert newf.sequence == 'CCTT'
        assert newf.quality == 'BC;;'

    def test_qual(self):
        self.f.quality = '!@;A'
        self.f.to_qual()
        assert self.f.to_qual().split("\n")[1] == '0 31 26 32'

    #@attr('fixme')
    #def test_read_and_plot(self):
    #    self.f.read(self.f._multiple_fastq_example)
    #    self.f.plot()

    def test_clear(self):
        self.f.clear()




from biokit import Sequence, DNA



def test():
    s1 = Sequence('ACGT')
    s2 = Sequence('AAAA')
    s1 + s2
    s2 + s1
    s1+= s2
    "aa" + s1

    try:
        s1 += 2
        assert False
    except:
        assert True

    s1.histogram()
    s1.pie() 

    d1 = Sequence('GAGCCTACTAACGGGAT')
    d2 = Sequence('CATCGTAATGACGGCCT')
    #d1 = Sequence(d2)
    assert d1.hamming_distance(d2) == 7
                   
    d1.lower()
    assert d1.sequence == 'gagcctactaacgggat'
    d2.upper()


    d1._check_sequence()

    # __repr__ and __str__
    print(d1)
    d1.__repr__()
    d1 = Sequence('AAAAa') # length greater than 10
    print(d1)
    d1.__repr__()


    d1 = Sequence('ACGT')
    dna = DNA('ACGT')
    try:
        d1 += rna
        assert False
    except:
        assert True
    try:
        dna += d1
        assert False
    except:
        assert True


    s = Sequence(DNA('ACGT'))
    assert s == 'ACGT'
    assert s == dna



from biokit import Taxonomy
from nose.plugins.attrib import attr



@attr("skiptravis")
def test_taxonomy():
    t = Taxonomy()
    t.load_records()
    lineage = t.get_lineage(9606)
    assert len(lineage) == 31
    assert 'Mammalia' in lineage
    assert t.fetch_by_id('10090')['name'] == 'Mus musculus' 
    ret = t.fetch_by_name('Mus Musculus')
    assert ret[0]['id'] == '10090'

    lineage = t.get_lineage_and_rank(9606)

    tree = t.get_family_tree(9606)





from biokit.rtools import session
from nose.plugins.attrib import attr

@attr('Ronly')
def test_session():

    sess = session.RSession()
    print(sess)
    sess.get_version()
    try:
        sess.reconnect()
        assert False
    except:
        assert True


@attr('Ronly')
def test_attribute():
    s = session.RSession()
    s.run("b=1")
    assert 1 == s.b
    try:
        s.c
        assert False
    except:
        assert True
    assert 1 == s.b





import os



class CreateDummy(object):

    def __init__(self, directory="dummy"):
        self.directory = directory
        try:
            os.mkdir(directory)
        except:
            pass

    def __call__(self):
        try:
            os.mkdir(directory)
            self.created = True
        except:
            self.created = False
        self.create_directory()
        self.create_description()
        self.create_namespace()
        self.create_manual()
        self.create_R_file()
    
        # Create the tar ball file
        code = "cd %s; R CMD build %s ; cd .." % (self.directory, '.')
        os.system(code)

    def _clean(self):
        import shutil
        # we could just used the last command but this adds some safety
        shutil.rmtree(os.sep.join([self.directory, 'man']))
        shutil.rmtree(os.sep.join([self.directory, 'R']))
        shutil.rmtree(os.sep.join([self.directory]))

    def create_directory(self):
        try:
            os.mkdir(os.sep.join([self.directory, 'man']))
        except:
            pass
        try:
            os.mkdir(os.sep.join([self.directory, 'R']))
        except:
            pass

    def create_manual(self):
        package = r"""
\name{dummy_test-package}
\alias{dummy_test-package}
\alias{dummy_test}
\docType{package}
\title{R version of dummy_test}
\description{}
\details{}
\author{BadLuck, Maintainer: Any. Volunteer <dummy@dummy.du>}
\keyword{ package }
\examples{
    library(dummy_test)
        dummy_test()
}"""        
        filename = os.sep.join([self.directory, "man", "dummytest-package.Rd"])
        with open(filename, 'w') as fh:
            fh.write(package)

        function= r"""
\name{dummy_test}
\alias{dummy_test}
\alias{dummy_test}
\title{whatever}
\description{whatever}
\usage{dummy_test()}
\arguments{}
\author{whoever}
\examples{
library(dummytest)
dummy_test()
}
        """
        filename = os.sep.join([self.directory, "man", "dummy_test.Rd"])
        with open(filename, 'w') as fh:
            fh.write(package)
    
    def create_R_file(self):
        code = """dummy_test <- function(){
                print("This is the dummytest package")
                }"""
        filename = os.sep.join([self.directory, "R", "dummy_test.R"])
        with open(filename, 'w') as fh:
            fh.write(code)

    def create_namespace(self):
        filename = os.sep.join([self.directory, "NAMESPACE"])
        with open(filename, "w") as fh:
            fh.write("""export("dummy_test")""")

    def create_description(self):
        description = """Package: dummytest
Type: Package
Title: A wonderful dummy package for testing
Version: 1.0.0
Date: 2014-03-13
Author: BackLuck
Maintainer: Any.Volunteer <dummy@dummy.du>
Depends: R (>= 2.15.0), methods
Suggests: 
Description: This package can be install and remove without interferin with the user packages in principle except if a smart developer decides to call its package dummytest.
License: GPL-3
LazyLoad: yes
SystemRequirements: """
        filename = os.sep.join([self.directory, "DESCRIPTION"])
        with open(filename, 'w') as fh:
            fh.write(description)









def test1():
    from biokit.viz import scatter_hist
    scatter_hist(x=[1,2,3,4], y=[3,5,6,4],
        kargs_scatter={
            's':[200,400,600,800],
            'c': ['red', 'green', 'blue', 'yellow'],
            'alpha':0.5},
        kargs_histx={'color': 'red'},
        kargs_histy={'color': 'green'})

    from biokit.viz import ScatterHist

    s = ScatterHist(x=[1,2,3,4], y=[3,4,5,6])
    s.plot()
    s.plot(scatter_position='top right')
    s.plot(scatter_position='top left')
    s.plot(scatter_position='bottom right')



from biokit.viz import imshow, Imshow



def test_viz_imshow():
    from pandas import DataFrame
    df = DataFrame({'a':[1,2], 'b':[3,4]})
    imshow(df)

def test_viz_imshow_class():
    from pandas import DataFrame
    df = DataFrame({'a':[1,2], 'b':[3,4]})
    im = Imshow(df)
    im.plot(xticks_on=False, yticks_on=False)



from biokit.rtools import tools
from nose.plugins.attrib import attr



def test_codecs():

    assert 'T' == tools.bool2R(True)
    assert 'F' == tools.bool2R(False)
    try:
        tools.bool2R('ggg')
        assert False
    except:
        assert True

@attr('Ronly')
def test_rcode():
    r = tools.rcode('a=1')
    assert r.a == 1 




from biokit.stats import mixture
import pandas as pd
from easydev import Progress
import numpy as np

data = pd.read_csv("test.csv", index_col=0, header=None)


# Generate a random mixture model

# ############################################### Create data set to be tested
# symetric case
def get_data_case1():
    return mixture.GaussianMixture(mu=[-1, 0, 1], sigma=[0.3,0.3,0.3], 
        mixture=[0.1,.8,.1], N=100000)

# asymetric. Note mu slightly shifted to the left
class Data(object):
    def __init__(self):
        self.mus = [8.8, 10, 11]
        self.sigmas = [0.3, 0.3, 0.3]
        self.mixtures = [.2, .7, .1]
        self.N = 100000

    def __call__(self):
        xx =  mixture.GaussianMixture(
            mu=self.mus, 
            sigma=self.sigmas,
            mixture=self.mixtures, N=self.N)
        return xx.data


class Data2(Data):
    def __init__(self):
        self.mus = [9, 10, 11]
        self.mixture = [.1, .8, .1]


class Benchmark(object):

    def __init__(self):
        self.generator = Data()
        self.N = 100
        self.k = 3

    def run_biokit(self):
        results = {"mus":[], "sigmas": [], "pis": []}
        pb = Progress(self.N)
        for i in range(self.N):
            data = self.generator()
            em = mixture.EM(data)
            em.estimate(k=self.k)

            results['mus'].append(em.results['mus'])
            results["sigmas"].append(em.results['sigmas'])
            results["pis"].append(em.results['pis'])
            pb.animate(i+1) 
        self.results_biokit = results 

    def run_biokit2(self):
        results = {"mus":[], "sigmas": [], "pis": []}
        pb = Progress(self.N)
        for i in range(self.N):
            data = self.generator()
            em = mixture.GaussianMixtureFitting(data)
            em.estimate(k=self.k)
  
            mus = em.results['mus']
            pis = em.results['pis']
            sigmas = em.results['sigmas']
            indices = np.argsort(mus)

            results['mus'].append([mus[x] for x in indices])
            results["sigmas"].append([sigmas[x] for x in indices])
            results["pis"].append([pis[x] for x in indices])
            pb.animate(i+1)
        self.results_biokit2 = results 

    def run_gmm(self):
        from sklearn.mixture import gmm

        results = {"mus":[], "sigmas": [], "pis": []}
        pb = Progress(self.N)
        g = gmm.GMM(n_components=self.k)
        for i in range(self.N):
            g.fit(np.matrix(self.generator()).T)
            
            pis = g.weights_
            mus = g.means_.T[0]
            sigmas = np.sqrt(g.covars_.T[0])

            indices = np.argsort(mus)

            results['pis'].append([pis[x] for x in indices])
            results['mus'].append([mus[x] for x in indices])
            results['sigmas'].append([sigmas[x] for x in indices])
            pb.animate(i+1) 

        self.results_sklearn = results


















from biokit import DNA


def test_dna():
    s1 = DNA('ACGT')
    s2 = DNA('AAAA')
    s1+s2
    s1+=s2

    assert len(s1) == 8
    assert s1._data == 'ACGTAAAA'


    s1.get_complement()
    s1.complement

    s1.get_reverse_complement()
    
    s1.gc_content()
    s1.get_rna()

    empty = DNA()
    empty.gc_content()





# -*- python -*-
#
#  This file is part of biokit software
#
#  Copyright (c) 2014-
#
#  File author(s): Thomas Cokelaer <cokelaer@ebi.ac.uk>
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://github.com/biokit
#
##############################################################################
from easydev import check_param_in_list


class Py2R(object):
    """Simple class to convert Python object to R objects as strings"""
    @staticmethod
    def from_bool(self, value):
        check_param_in_list(value, [True, False])
        if value is True:
            return "T"
        if value is False:
            return "F"

    def from_dict(self, value):
        return('list(' + ','.join(['%s=%s' % (self.Str4R(a[0]), self.Str4R(a[1])) 
            for a in value.items()]) + ')')

    def Str4R(self, obj):
        """convert a Python basic object into an R object in the form of string."""
        #return str_func.get(type(obj), OtherStr)(obj)
        # for objects known by PypeR
        if type(obj) in str_func:
            return(str_func[type(obj)](obj))

        # for objects derived from basic data types
        for tp in base_tps:
            if isinstance(obj, tp):
                return(str_func[tp](obj))
        # for any other objects
        return(OtherStr(obj))





import numpy as np
import scipy.stats

from mixem.distribution.distribution import Distribution


class PoissonDistribution(Distribution):
    """Univariate normal distribution with parameters (mu, sigma)."""

    def __init__(self, lmbda, loc):
        self.lmbda = lmbda
        self.loc = 0

    def log_density(self, data):
        assert(len(data.shape) == 1), "Expect 1D data!"

        return - self.lmbda  + data * np.log(self.lmbda)  

        #res = scipy.stats.poisson.pmf(data, self.lmbda, loc=self.sigma)

    def estimate_parameters(self, data, weights):
        assert(len(data.shape) == 1), "Expect 1D data!"

        wsum = np.sum(weights)

        # derivative over lambda of -lambda + x log(lamnda) 
        self.lmbda = np.sum(weights * data) / wsum #- self.loc / self.lmbda


    def __repr__(self):
        return "Poisson[lmbda={lmbda:.4g}, loc={loc:.4g}]".format(lmbda=self.lmbda, loc=self.loc)





#! /usr/bin/env python
"""
Author: Jeremy M. Stober
Program: NORMAL.PY
Date: Friday, July 7, 2011
Description: Manipulating normal distributions.
"""

import numpy as np
import numpy.linalg as la
import numpy.random as npr
import random as pr
npa = np.array
ix  = np.ix_ # urgh - sometimes numpy is ugly!

class Normal(object):
    """
    A class for storing the parameters of a multivariate normal
    distribution. Supports evaluation, sampling, conditioning and
    marginalization.
    """

    def __init__(self, dim, mu = None, sigma = None, data = None,
                 parent = None, cond = None, margin = None):
        """
        Initialize a normal distribution.

        Parameters
        ----------
        dim : int
            Number of dimensions (e.g. number of components in the mu parameter).
        mu : array, optional
            The mean of the normal distribution.
        sigma : array, optional
            The covariance matrix of the normal distribution.
        data : array, optional
            If provided, the parameters of the distribution will be estimated from the data. Rows are observations, columns are components.
        parent : Normal, optional
            A reference to a parent distribution that was marginalized or conditioned.
        cond : dict, optional
            A dict of parameters describing how the parent distribution was conditioned.
        margin : dict, optional
            A dict of parameters describing how the parent distribution was marginalized.

        Examples
        --------
        >>> x = Normal(2,mu = np.array([0.1,0.7]), sigma = np.array([[ 0.6,  0.4], [ 0.4,  0.6]]))
        >>> print x
        [ 0.1  0.7]
        [[ 0.6  0.4]
        [ 0.4  0.6]]

        To condition on a value (and index):
        
        >>> condx = x.condition([0],0.1)
        >>> print condx
        [ 0.7]
        [[ 0.33333333]]
        
        """

        self.dim = dim # full data dimension

        if not mu is None  and not sigma is None:
            pass
        elif not data is None:
            # estimate the parameters from data - rows are samples, cols are variables
            mu, sigma = self.estimate(data)
        else:
            # generate random means
            mu = npr.randn(dim)
            sigma = np.eye(dim)

        self.cond = cond
        self.margin = margin
        self.parent = parent

        self.update(npa(mu),npa(sigma))


    def update(self, mu, sigma):
        """
        Update the distribution with new parameters.

        Parameters
        ----------
        mu : array
            The new mean parameters.
        sigma : array
            The new covariance matrix.

        Example
        -------

        >>> x = Normal(2,mu = np.array([0.1,0.7]), sigma = np.array([[ 0.6,  0.4], [ 0.4,  0.6]]))
        >>> print x
        [ 0.1  0.7]
        [[ 0.6  0.4]
        [ 0.4  0.6]]

        >>> x.update(np.array([0.0,0.0]), x.E)
        >>> print x
        [ 0.0  0.0]
        [[ 0.6  0.4]
        [ 0.4  0.6]]
        """

        self.mu = mu
        self.E = sigma

        det = None
        if self.dim == 1:
            self.A = 1.0 / self.E
            det = np.fabs(self.E[0])
        else:
            self.A = la.inv(self.E) # precision matrix
            det = np.fabs(la.det(self.E))

        self.factor = (2.0 * np.pi)**(self.dim / 2.0) * (det)**(0.5)

    def __str__(self):
        return "%s\n%s" % (str(self.mu), str(self.E))

    def mean(self):
        return self.mu

    def covariance(self):
        return self.E

    def pdf(self, x):
        dx = x - self.mu
        A = self.A
        fE = self.factor

        return np.exp(-0.5 * np.dot(np.dot(dx,A),dx)) / fE

    def pdf_mesh(self, x, y):
        # for 2d meshgrids
        # use matplotlib.mlab.bivariate_normal -- faster (vectorized)

        z = np.zeros((len(y),len(x)))
        
        for (i,v) in enumerate(x):
            for (j,w) in enumerate(y):
                z[j,i] = self.pdf([v,w])
        
        return z

    def simulate(self, ndata = 100):
        """
        Draw pts from the distribution.
        """
        return npr.multivariate_normal(self.mu, self.E, ndata)

    def estimate(self, data):
        mu = np.mean(data, axis=0)
        sigma = np.cov(data, rowvar=0)
        return mu, sigma

    def marginalize(self, indices):
        """
        Creates a new marginal normal distribution for ''indices''.
        """
        indices = npa(indices)
        return Normal(len(indices), mu = self.mu[indices], sigma = self.E[ix(indices,indices)], margin = {'indices' : indices}, parent = self)

    def condition(self, indices, x):
        """
        Creates a new normal distribution conditioned on the data x at indices.
        """

        idim = indices
        odim = npa([i for i in range(self.dim) if not i in indices])

        Aaa = self.A[ix(odim,odim)]
        Aab = self.A[ix(odim,idim)]
        iAaa = None
        det = None

        if len(odim) == 1: # linalg does not handle d1 arrays
            iAaa = 1.0 / Aaa
            det = np.fabs(iAaa[0])
        else:
            iAaa = la.inv(Aaa)
            det = np.fabs(la.det(iAaa))

        # compute the new mu
        premu = np.dot(iAaa, Aab)

        mub = self.mu[idim]
        mua = self.mu[odim]
        new_mu = mua - np.dot(premu, (x - mub))

        new_E = iAaa
        return Normal(len(odim), mu = new_mu, sigma = new_E,
                      cond = {'data' : x, 'indices' : indices},
                      parent = self)




from biokit.viz import corrplot
import string
import pandas as pd
import numpy as np
import pylab


class TestCorrplot(object):

    @classmethod
    def setup_class(klass):
        try:
            letters = string.uppercase[0:10]
        except: #python3
            letters= string.ascii_uppercase[0:10]
        df = pd.DataFrame(dict(( (k, np.random.random(10)+ord(k)-65) for k in letters)))
        klass.s = corrplot.Corrplot(df.corr())
        klass.s = corrplot.Corrplot(df)

    def test_correlation(self):
        df1 = pd.DataFrame([[1,2,3,4],[4,5,1,2]])
        c1 = corrplot.Corrplot(df1)

        df2 = pd.DataFrame([[1,2,3,4],[4,5,1,2]]).corr()
        c2 = corrplot.Corrplot(df2)
        
        # in c1, the correlation is computed.
        assert (c1.df == c2.df).all().all() == True
        
    def test_plot_square(self):
        self.s.plot(colorbar=False, method='square', shrink=.9, rotation=45)

    def test_plot_text(self):
        self.s.plot(method='text', fontsize=8)

    def test_plot_color(self):
        self.s.plot(method='color')

    def test_order(self):
        self.s.order(method='single', inplace=True)
        self.s.plot(method='pie', shrink=.8, order_metric='jaccard')
    
    def test_figure(self):
        self.s.plot(fig=1)
        self.s.plot(fig=None)
        fig = pylab.clf()
        self.s.plot(fig=fig)

        ax = pylab.subplot(2,1,2)
        self.s.plot(fig=fig, ax=ax)


    def test_lower(self):
        self.s.plot(colorbar=False, method='circle', shrink=.9, lower='circle'  )

    def test_upper(self):
        self.s.plot(colorbar=False, method='circle', shrink=.9, upper='circle'  )

    def test_both(self):
        self.s.plot(colorbar=False, method='circle', shrink=.9, upper='circle', lower='ellipse')
        
    def test_wrong_method(self):
        try:
            self.s.plot(colorbar=False, method='dummy')
            assert False
        except ValueError:
            assert True



from biokit.network.complexes import Complexes
from nose.plugins.attrib import attr



@attr("skiptravis")
def test_complexes():
    c = Complexes(cache=True)
    c.remove_homodimers()
    and_gates, subset = c.search_complexes(['P54289'])
    and_gates, subset = c.search_complexes(['P54289'], verbose=True)
    assert len(subset)


    c.search('P54289')
    c.hist_participants()
    c.stats()


    # let us find a complex with all its participants
    user_species = ['Q13936', u'Q08289', u'P54289']
    and_gates, subset = c.search_complexes(user_species)
    assert len(and_gates)
    assert 'EBI-9687812' in and_gates.keys()


    assert c.uniprot2genename('Q13936') == ['CACNA1C CACH2 CACN2 CACNL1A1 CCHL1A1']
    try:
        assert c.uniprot2genename('dummy')
        assert False
    except:
        assert True


    assert c.chebi2name('CHEBI:18420') == 'magnesium(2+)'
    







def test1():
    from numpy import random
    from biokit.viz import hist2d
    X = random.randn(10001)
    Y = random.randn(10001)
    h = hist2d.Hist2D(X,Y)
    h.plot(bins=100, contour=True, norm='log', normed=True)
    h.plot(bins=[100,100], contour=True, norm='log', normed=True)




from biokit.stats.mixture import GaussianMixtureModel
from biokit.stats.mixture import GaussianMixture
from biokit.stats.mixture import GaussianMixtureFitting




def test_gmm():
    gmm = GaussianMixtureModel(k=2)
    gmm.pdf(5, params=[-1,0.5,0.2,1,0.5,0.8])

def test_gmf():
    m = GaussianMixture(mu=[-2,1], sigma=[0.5,0.5], mixture=[.2,.8], N=50)
    mf = GaussianMixtureFitting(m.data)
    mf.estimate()
    mf.k = 3
    mf.estimate()
    assert mf.k == 3
    #mf.model = 'CG'
    #mf.estimate()

def test_em():
    from biokit.stats.mixture import EM
    m = GaussianMixture(mu=[-2,1], sigma=[0.5,0.5], mixture=[.2,.8], N=50)
    em = EM(m.data)
    em.estimate()
    em.plot()

def test_amf():
    from biokit.stats.mixture import AdaptativeMixtureFitting
    m = GaussianMixture(mu=[-2,1], sigma=[0.5,0.5], mixture=[.2,.8], N=50)
    amf = AdaptativeMixtureFitting(m.data)
    amf.run(kmin=1,kmax=4)


    amf.diagnostic()



__version__ = "0.1.2"
import pkg_resources
try:
    version = pkg_resources.require(biokit)[0].version
except:
    version = __version__

# Creates the data directory if it does not exist
from easydev import CustomConfig
biokitPATH = CustomConfig("biokit").user_config_dir


from biokit import viz
from biokit import io
from biokit import services

from biokit.viz import *
from biokit.services import EUtils, KEGG, UniProt

from biokit import stats
from biokit.network import *

from biokit import sequence
from biokit.sequence import *

from biokit import goid
from biokit.goid import *

from biokit import taxonomy
from biokit.taxonomy import Taxonomy



def biokit_data(filename, where=None):
    """Simple utilities to retrieve data sets from biokit/data directory"""
    import os
    import easydev
    sequana_path = easydev.get_package_location('biokit')
    share = os.sep.join([sequana_path , "biokit", 'data'])
    # in the code one may use / or \ 
    if where:
        filename = os.sep.join([share, where, filename])
    else:
        filename = os.sep.join([share, filename])
    if os.path.exists(filename) is False:
        raise Exception('unknown file %s' % filename)
    return filename





from biokit import RNA


def test_rna():
    s1 = RNA('ACGU')
    s2 = RNA('AAAA')
    s1+s2
    s1+=s2

    assert len(s1) == 8
    assert s1._data == 'ACGUAAAA'


    s1.get_complement()
    s1.complement

    s1.get_reverse_complement()
    
    s1.gc_content()
    s1.get_dna()

    empty = RNA()
    empty.gc_content()
